@@ -1,7 +1,7 @@
-/*  $Id: weather-search.h 2366 2007-01-15 19:10:23Z nick $
+/*  $Id: weather-search.h 7588 2009-06-16 07:06:04Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -37,8 +37,6 @@ typedef struct
 
   gchar        *proxy_host;
   gint          proxy_port;
-
-  gchar        *recv_buffer;
 }
 search_dialog;
 
