@@ -1,4 +1,4 @@
-/*  $Id: weather.c 3231 2007-09-14 14:43:27Z nick $
+/*  $Id: weather.c 7588 2009-06-16 07:06:04Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -41,16 +41,21 @@
 #define XFCEWEATHER_ROOT "weather"
 #define UPDATE_TIME      1600
 #define BORDER           8
-#define PLUGIN_WEBSITE   "http://goodies.xfce.org/projects/panel-plugins/xfce4-weather-plugin"
-
 
+#if !GLIB_CHECK_VERSION(2,14,0)
+#define g_timeout_add_seconds(t,c,d) g_timeout_add((t)*1000,(c),(d))
+#endif
 
 gboolean
 check_envproxy (gchar **proxy_host,
                 gint   *proxy_port)
 {
 
-  gchar *env_proxy = getenv ("HTTP_PROXY"), *tmp, **split;
+  gchar *env_proxy = NULL, *tmp, **split;
+
+  env_proxy = getenv ("HTTP_PROXY");
+  if (!env_proxy)
+    env_proxy = getenv ("http_proxy");
 
   if (!env_proxy)
     return FALSE;
@@ -88,10 +93,12 @@ check_envproxy (gchar **proxy_host,
 
 
 static gchar *
-make_label (xml_weather *weatherdata,
-            datas        opt,
-            units        unit,
-            gint         size)
+make_label (xml_weather    *weatherdata,
+            datas          opt,
+            units          unit,
+            gint           size,
+	    GtkOrientation orientation,
+	    gboolean       multiple_labels)
 {
 
   gchar       *str, *value;
@@ -138,7 +145,9 @@ make_label (xml_weather *weatherdata,
     }
 
   /* arbitrary, choose something that works */
-  if (size > 36)
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    txtsize = "medium";
+  else if (size > 36)
     txtsize = "medium";
   else if (size > 30)
     txtsize = "small";
@@ -167,19 +176,31 @@ make_label (xml_weather *weatherdata,
     }
 
 
-
-  if (value != NULL)
-    {
-      str = g_strdup_printf ("<span size=\"%s\">%s: %s</span>",
-                             txtsize, lbl, value);
-      g_free (value);
-    }
-  else
-    {
-      str = g_strdup_printf ("<span size=\"%s\">%s: %s %s</span>",
-                             txtsize, lbl, rawvalue, get_unit (unit, opt));
-    }
-
+  if (multiple_labels) {
+    if (value != NULL)
+      {
+	str = g_strdup_printf ("<span size=\"%s\">%s: %s</span>",
+                               txtsize, lbl, value);
+	g_free (value);
+      }
+    else
+      {
+	str = g_strdup_printf ("<span size=\"%s\">%s: %s %s</span>",
+                               txtsize, lbl, rawvalue, get_unit (unit, opt));
+      }
+  } else {
+    if (value != NULL)
+      {
+	str = g_strdup_printf ("<span size=\"%s\">%s</span>",
+                               txtsize, value);
+	g_free (value);
+      }
+    else
+      {
+	str = g_strdup_printf ("<span size=\"%s\">%s %s</span>",
+                               txtsize, rawvalue, get_unit (unit, opt));
+      }
+  }
   return str;
 }
 
@@ -203,7 +224,10 @@ set_icon_error (xfceweather_data *data)
     }
 
   /* arbitrary, choose something that works */
-  if (size > 36)
+
+  if (data->orientation == GTK_ORIENTATION_HORIZONTAL)
+    txtsize = "medium";
+  else if (size > 36)
     txtsize = "medium";
   else if (size > 30)
     txtsize = "small";
@@ -220,15 +244,20 @@ set_icon_error (xfceweather_data *data)
 
   gtk_widget_get_size_request (data->scrollbox, NULL, &height);
 
-  icon = get_icon ("99", data->size - height - 2);
+  if (data->orientation == GTK_ORIENTATION_VERTICAL)
+    icon = get_icon ("99", data->size - height - 2);
+  else
+    icon = get_icon ("99", data->size);
 
   gtk_image_set_from_pixbuf (GTK_IMAGE (data->iconimage), icon);
 
   if (G_LIKELY (icon))
     g_object_unref (G_OBJECT (icon));
 
+#if !GTK_CHECK_VERSION(2,12,0)
   gtk_tooltips_set_tip (data->tooltips, data->tooltipbox,
                         _("Cannot update weather data"), NULL);
+#endif
 }
 
 
@@ -246,7 +275,7 @@ set_icon_current (xfceweather_data *data)
     {
       opt = g_array_index (data->labels, datas, i);
 
-      str = make_label (data->weatherdata, opt, data->unit, data->size);
+      str = make_label (data->weatherdata, opt, data->unit, data->size, data->orientation, (data->labels->len > 1));
 
       gtk_scrollbox_set_label (GTK_SCROLLBOX (data->scrollbox), -1, str);
 
@@ -262,7 +291,10 @@ set_icon_current (xfceweather_data *data)
   else
     {
       gtk_widget_get_size_request (data->scrollbox, NULL, &height);
-      size = data->size - height - 2;
+      if (data->orientation == GTK_ORIENTATION_VERTICAL)
+        size = data->size - height - 2;
+      else
+        size = data->size;
     }
 
   icon = get_icon (get_data (data->weatherdata, WICON), size);
@@ -272,9 +304,11 @@ set_icon_current (xfceweather_data *data)
   if (G_LIKELY (icon))
     g_object_unref (G_OBJECT (icon));
 
+#if !GTK_CHECK_VERSION(2,12,0)
   gtk_tooltips_set_tip (data->tooltips, data->tooltipbox,
                         translate_desc (get_data (data->weatherdata, TRANS)),
                         NULL);
+#endif
 }
 
 
@@ -338,9 +372,10 @@ update_weatherdata (xfceweather_data *data)
     }
 
   /* build url */
-  url = g_strdup_printf ("/weather/local/%s?cc=*&dayf=%d&unit=%c",
+  url = g_strdup_printf ("/weather/local/%s?cc=*&dayf=%d&unit=%c&link=xoap&prod=xoap&par=%s&key=%s",
                          data->location_code, XML_WEATHER_DAYF_N,
-                         data->unit == METRIC ? 'm' : 'i');
+                         data->unit == METRIC ? 'm' : 'i',
+			 PARTNER_ID, LICENSE_KEY);
 
   /* start receive thread */
   weather_http_receive_data ("xoap.weather.com", url, data->proxy_host,
@@ -450,6 +485,11 @@ xfceweather_read_config (XfcePanelPlugin  *plugin,
       data->proxy_port = data->saved_proxy_port;
     }
 
+  data->animation_transitions = xfce_rc_read_bool_entry (rc, 
+  		"animation_transitions", TRUE);
+
+  gtk_scrollbox_set_animate(GTK_SCROLLBOX(data->scrollbox), data->animation_transitions);
+
   data->labels = labels_clear (data->labels);
 
   for (i = 0; i < 100 /* arbitrary */ ; ++i)
@@ -504,6 +544,8 @@ xfceweather_write_config (XfcePanelPlugin  *plugin,
       xfce_rc_write_int_entry (rc, "proxy_port", data->proxy_port);
     }
 
+  xfce_rc_write_bool_entry (rc, "animation_transitions", data->animation_transitions);
+
   for (i = 0; i < data->labels->len; i++)
     {
       g_snprintf (label, 10, "label%d", i);
@@ -526,7 +568,7 @@ update_weatherdata_with_reset (xfceweather_data *data)
   update_weatherdata (data);
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
+    g_timeout_add_seconds (UPDATE_TIME, (GSourceFunc) update_weatherdata, data);
 }
 
 
@@ -549,6 +591,28 @@ close_summary (GtkWidget *widget,
 
 
 
+static void
+forecast_click (GtkWidget *widget,
+          gpointer   user_data)
+{
+  xfceweather_data *data = (xfceweather_data *) user_data;
+
+  if (data->summary_window != NULL)
+    {
+
+      gtk_window_present (GTK_WINDOW (data->summary_window));
+    }
+  else
+    {
+      data->summary_window = create_summary_window (data->weatherdata,
+                                                    data->unit);
+      g_signal_connect (G_OBJECT (data->summary_window), "destroy",
+                	G_CALLBACK (close_summary), data);
+
+      gtk_widget_show_all (data->summary_window);
+    }
+}
+
 static gboolean
 cb_click (GtkWidget      *widget,
           GdkEventButton *event,
@@ -558,21 +622,7 @@ cb_click (GtkWidget      *widget,
 
   if (event->button == 1)
     {
-
-      if (data->summary_window != NULL)
-        {
-
-          gtk_window_present (GTK_WINDOW (data->summary_window));
-        }
-      else
-        {
-          data->summary_window = create_summary_window (data->weatherdata,
-                                                        data->unit);
-          g_signal_connect (G_OBJECT (data->summary_window), "destroy",
-                            G_CALLBACK (close_summary), data);
-
-          gtk_widget_show_all (data->summary_window);
-        }
+      forecast_click(widget, user_data);
     }
   else if (event->button == 2)
     {
@@ -582,8 +632,6 @@ cb_click (GtkWidget      *widget,
   return FALSE;
 }
 
-
-
 static void
 mi_click (GtkWidget *widget,
           gpointer   user_data)
@@ -665,47 +713,87 @@ xfceweather_create_options (XfcePanelPlugin  *plugin,
   gtk_widget_show (dlg);
 }
 
+static gboolean weather_get_tooltip_cb (GtkWidget        *widget,
+					gint              x,
+					gint              y,
+					gboolean          keyboard_mode,
+					GtkTooltip       *tooltip,
+					xfceweather_data *data) 
+{
+  GdkPixbuf *icon;
+  gchar *markup_text;
+  
+  if (data->weatherdata == NULL) {
+    gtk_tooltip_set_text (tooltip, _("Cannot update weather data"));
+  } else {
+    markup_text = g_markup_printf_escaped(
+  	  "<b>%s</b>\n"
+	  "%s", 
+	  get_data (data->weatherdata, DNAM),
+	  translate_desc (get_data (data->weatherdata, TRANS))
+	  );
+    gtk_tooltip_set_markup (tooltip, markup_text);
+    g_free(markup_text);
+  }
+  icon = get_icon (get_data (data->weatherdata, WICON), 32);
+  gtk_tooltip_set_icon (tooltip, icon);
+  g_object_unref (G_OBJECT(icon));
+  
+  return TRUE;
+}
 
 
 static xfceweather_data *
 xfceweather_create_control (XfcePanelPlugin *plugin)
 {
   xfceweather_data *data = panel_slice_new0 (xfceweather_data);
-  GtkWidget        *vbox, *refresh;
+  GtkWidget        *refresh, *forecast;
   datas             lbl;
   GdkPixbuf        *icon = NULL;
 
   data->plugin = plugin;
 
+#if !GTK_CHECK_VERSION(2,12,0)
   data->tooltips = gtk_tooltips_new ();
   g_object_ref (data->tooltips);
   gtk_object_sink (GTK_OBJECT (data->tooltips));
-
+#endif
   data->scrollbox = gtk_scrollbox_new ();
 
   icon = get_icon ("99", 16);
   data->iconimage = gtk_image_new_from_pixbuf (icon);
-  gtk_misc_set_alignment (GTK_MISC (data->iconimage), 0.5, 1);
 
   if (G_LIKELY (icon))
     g_object_unref (G_OBJECT (icon));
 
   data->labels = g_array_new (FALSE, TRUE, sizeof (datas));
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  data->vbox_center_scrollbox = gtk_vbox_new(FALSE, 0);
+  data->top_hbox = gtk_hbox_new (FALSE, 0);
+  gtk_misc_set_alignment (GTK_MISC (data->iconimage), 1, 0.5);
+  gtk_box_pack_start (GTK_BOX (data->top_hbox), data->iconimage, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (data->vbox_center_scrollbox), data->scrollbox, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (data->top_hbox), data->vbox_center_scrollbox, TRUE, FALSE, 0);
 
-  gtk_box_pack_start (GTK_BOX (vbox), data->iconimage, TRUE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), data->scrollbox, TRUE, TRUE, 0);
+  data->top_vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (data->top_vbox), data->top_hbox, TRUE, FALSE, 0);
 
   data->tooltipbox = gtk_event_box_new ();
-  gtk_container_add (GTK_CONTAINER (data->tooltipbox), vbox);
+  gtk_container_add (GTK_CONTAINER (data->tooltipbox), data->top_vbox);
   gtk_widget_show_all (data->tooltipbox);
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET(data->tooltipbox), GTK_NO_WINDOW);
 
+#if GTK_CHECK_VERSION(2,12,0)
+  g_object_set (G_OBJECT(data->tooltipbox), "has-tooltip", TRUE, NULL);
+  g_signal_connect(G_OBJECT(data->tooltipbox), "query-tooltip", 
+		   G_CALLBACK(weather_get_tooltip_cb),
+		  data);
+#endif
   xfce_panel_plugin_add_action_widget (plugin, data->tooltipbox);
 
   g_signal_connect (G_OBJECT (data->tooltipbox), "button-press-event",
                     G_CALLBACK (cb_click), data);
+  gtk_widget_add_events(data->scrollbox, GDK_BUTTON_PRESS_MASK);
 
   /* add refresh button to right click menu, for people who missed the middle mouse click feature */
   refresh = gtk_image_menu_item_new_from_stock ("gtk-refresh", NULL);
@@ -716,6 +804,20 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
 
   xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (refresh));
 
+  /* add refresh button to right click menu, for people who missed the middle mouse click feature */
+  forecast = gtk_image_menu_item_new_with_mnemonic (_("_Forecast"));
+  icon = get_icon ("32", 16);
+  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(forecast),
+  	gtk_image_new_from_pixbuf(icon));
+  if (G_LIKELY (icon))
+    g_object_unref (G_OBJECT (icon));
+  gtk_widget_show (forecast);
+
+  g_signal_connect (G_OBJECT (forecast), "activate",
+                    G_CALLBACK (forecast_click), data);
+
+  xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (forecast));
+
   /* assign to tempval because g_array_append_val () is using & operator */
   lbl = TEMP;
   g_array_append_val (data->labels, lbl);
@@ -730,7 +832,7 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
   gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
+    g_timeout_add_seconds (UPDATE_TIME, (GSourceFunc) update_weatherdata, data);
 
   return data;
 }
@@ -756,8 +858,10 @@ xfceweather_free (XfcePanelPlugin  *plugin,
   xmlCleanupParser ();
 
   /* Free Tooltip */
+#if !GTK_CHECK_VERSION(2,12,0)
   gtk_tooltips_set_tip (data->tooltips, data->tooltipbox, NULL, NULL);
   g_object_unref (G_OBJECT (data->tooltips));
+#endif
 
   /* Free chars */
   g_free (data->location_code);
@@ -789,6 +893,36 @@ xfceweather_set_size (XfcePanelPlugin  *panel,
   return TRUE;
 }
 
+static gboolean
+xfceweather_set_orientation (XfcePanelPlugin  *panel,
+                             GtkOrientation    orientation,
+                             xfceweather_data *data)
+{
+  GtkWidget *parent = gtk_widget_get_parent(data->vbox_center_scrollbox);
+
+  data->orientation = orientation;
+
+  g_object_ref(G_OBJECT(data->vbox_center_scrollbox));
+  gtk_container_remove(GTK_CONTAINER(parent), data->vbox_center_scrollbox);
+
+  if (data->orientation == GTK_ORIENTATION_HORIZONTAL) {
+    gtk_box_pack_start (GTK_BOX (data->top_hbox), data->vbox_center_scrollbox, TRUE, FALSE, 0);
+  } else {
+    gtk_box_pack_start (GTK_BOX (data->top_vbox), data->vbox_center_scrollbox, TRUE, FALSE, 0);
+  }
+  g_object_unref(G_OBJECT(data->vbox_center_scrollbox));
+
+  gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
+  
+  if (data->weatherdata)
+    set_icon_current (data);
+  else
+    set_icon_error (data);
+
+  /* we handled the orientation */
+  return TRUE;
+}
+
 
 
 static void
@@ -805,6 +939,7 @@ weather_construct (XfcePanelPlugin *plugin)
   xfceweather_set_visibility (data);
 
   xfceweather_set_size (plugin, xfce_panel_plugin_get_size (plugin), data);
+  xfceweather_set_orientation (plugin, xfce_panel_plugin_get_orientation(plugin), data);
 
   gtk_container_add (GTK_CONTAINER (plugin), data->tooltipbox);
 
@@ -817,6 +952,9 @@ weather_construct (XfcePanelPlugin *plugin)
   g_signal_connect (G_OBJECT (plugin), "size-changed",
                     G_CALLBACK (xfceweather_set_size), data);
 
+  g_signal_connect (G_OBJECT (plugin), "orientation-changed",
+                    G_CALLBACK (xfceweather_set_orientation), data);
+
   xfce_panel_plugin_menu_show_configure (plugin);
   g_signal_connect (G_OBJECT (plugin), "configure-plugin",
                     G_CALLBACK (xfceweather_create_options), data);
